/* ignore.c: routines for ignoring fools
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * Copyright (C) 2003 Matthew Kennedy
 */

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <db.h>

#include "ignore.h"
#include "gconfig.h"
#define db_create "db_create"

static DB *dbp;

int
ignore_init(void)
{
  int ret;

  ret = db_create(&dbp, NULL,0);
  if (ret != 0)
    {
      fprintf(stderr, "db_create: %s\n", db_strerror(ret));
      goto error_create;
    }
  ret = dbp->open(dbp, config_ignore, NULL, DB_BTREE, DB_CREATE, 0644);
  if (ret != 0)
    {
      fprintf(stderr, "db_open: %s\n", db_strerror(ret));
      goto error_open;
    }

  return 0;

 error_open:
 error_create:
  return IGNORE_ERROR;
}

ignore_ret_t
ignore_done(void)
{
  int ret;
  
  ret = dbp->close(dbp, 0);
  if (ret == 0)
    return IGNORE_OK;
  else 
    return IGNORE_ERROR;
}


ignore_ret_t
ignore_add(char *username)
{
  int ret;
  DBT key, data;

  memset(&key, 0, sizeof(key));
  memset(&data, 0, sizeof(data));
  key.data = username;
  key.size = strlen(username);
  data.data = "";
  data.size = 0;
  ret = dbp->put(dbp, NULL, &key, &data, DB_NOOVERWRITE);
  switch (ret)
    {
    case 0:
      return IGNORE_OK;
    case DB_KEYEXIST:
      return IGNORE_EXISTS;
    default:
      fprintf(stderr, "DB->put: %s\n", db_strerror(ret));
      goto error_put;
    }

 error_put:
  return IGNORE_ERROR;
}

ignore_ret_t
ignore_delete(char *username)
{
  DBT key;
  int ret;
  
  memset(&key, 0, sizeof(key));
  key.data = username;
  key.size = strlen(username);
  ret = dbp->del(dbp, NULL, &key, 0);
  if (ret != 0)
    {
      fprintf(stderr, "DB->del: %s\n", db_strerror(ret));
      goto error_del;
    }

  return IGNORE_OK;

 error_del:
  return IGNORE_ERROR;
}

ignore_ret_t
ignore_is_ignorable(char *username)
{
  int ret;
  DBT key, data;

  memset(&key, 0, sizeof(key));
  memset(&data, 0, sizeof(data));
  key.data = username;
  key.size = strlen(username);
  data.data = "";
  data.size = 0;
  ret = dbp->get(dbp, NULL, &key, &data, 0);
  switch (ret)
    {
    case 0:
      return IGNORE_IGNORABLE;
    case DB_NOTFOUND:
      return IGNORE_OK;
    default:
      fprintf(stderr, "DB->get: %s\n", db_strerror(ret));
      goto error_del;
    }

 error_del:
  return IGNORE_ERROR;
}
